'''
    flaskext.fulfil
    ---------------

    A Flask extension providing Fulfil.IO Api access.

    :copyright: (c) 2016 by Fulfil.IO Inc.
    :license: BSD, see LICENSE for more details.
'''
from fulfil_client import Client, BearerAuth
from utils import client_url


__version__ = '0.3.0'
__author__ = 'Fulfil.Io Inc.'
__license__ = 'BSD'
__copyright__ = '(c) 2016 by Fulfil.IO Inc.'
__all__ = ['Fulfil']


class Fulfil(object):
    """

    To get started you will wrap your application's app object something like
    this::

        app = Flask(__name__)
        fulfil = Fulfil(app)

    Configuration::

        FULFIL_SUBDOMAIN: the subdomain of your Fulfil account.
        FULFIL_API_KEY: The API_KEY to access Fulfil services.

    :param app: The Flask application object. Defaults to None.
    """
    client = None

    def __init__(self, app=None):
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        '''Initalizes the application with the extension.
        :param app: The Flask application object.
        '''
        offline_access_token = app.config.get('FULFIL_OFFLINE_ACCESS_TOKEN')
        if offline_access_token:
            self.client = Client(
                app.config['FULFIL_SUBDOMAIN'],
                auth=BearerAuth(offline_access_token)
            )
        else:
            self.client = Client(
                app.config['FULFIL_SUBDOMAIN'],
                app.config['FULFIL_API_KEY'],
            )
        app.jinja_env.filters['client_url'] = client_url

    def model(self, name):
        return self.client.model(name)

    def record(self, model_name, record_id):
        return self.client.record(model_name, record_id)



'''
Flask-Fulfil
============

Fulfil.IO for your Flask app
'''

from setuptools import setup

setup(
    name='Flask-Fulfil',
    version='0.3.0',
    url='https://github.com/fulfilio/flask-fulfil',
    license='BSD',
    author='Fulfil.IO Inc.',
    author_email='hello@fulfil.io',
    description='Fulfil.IO for Flask Apps',
    long_description=__doc__,
    packages=['flask_fulfil'],
    zip_safe=False,
    platforms='any',
    install_requires=['Flask', 'fulfil_client>=0.9.0', 'six'],
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
)





# -*- coding: utf-8 -*-
from six.moves.urllib import urlparse, urlencode
from math import ceil

from flask import current_app
from fulfil_client.client import dumps


def client_url(model, id=None, domain=None):
    """
    A filter for template engine to generate URLs that open
    on fulfil client of the instance::

        <a href="{{ 'product.product'|client_url }}">Product List</a>
        <a href="{{ 'product.product'|client_url(product.id) }}">{{ product.name }}</a>

    A more sophisticated example with filter

        <a href="{{ 'product.product'|client_url(domain=[('salable', '=', True)]) }}">
            Salable Products
        </a>
    """
    subdomain = current_app.config['FULFIL_SUBDOMAIN']
    url = 'https://{subdomain}.fulfil.io/client/#/model/{model}'.format(
        subdomain=subdomain,
        model=model
    )
    if id:
        url += '/%d' % id
    elif domain:
        url += '?' + urlencode({'domain': dumps(domain)})
    return url


def batch(iterable, n=1):
    l = len(iterable)
    for ndx in range(0, l, n):
        yield iterable[ndx:min(ndx + n, l)]


class Pagination(object):
    """Internal helper class returned by :meth:`BaseQuery.paginate`.  You
    can also construct it from any other SQLAlchemy query object if you are
    working with other libraries.  Additionally it is possible to pass `None`
    as query object in which case the :meth:`prev` and :meth:`next` will
    no longer work.
    """

    def __init__(self, page, per_page, total, items):
        #: the current page number (1 indexed)
        self.page = page
        #: the number of items to be displayed on a page.
        self.per_page = per_page
        #: the total number of items matching the query
        self.total = total
        #: the items for the current page
        self.items = items

    @property
    def pages(self):
        """The total number of pages"""
        if self.per_page == 0:
            pages = 0
        else:
            pages = int(ceil(self.total / float(self.per_page)))
        return pages

    @property
    def prev_num(self):
        """Number of the previous page."""
        return self.page - 1

    @property
    def has_prev(self):
        """True if a previous page exists"""
        return self.page > 1

    @property
    def has_next(self):
        """True if a next page exists."""
        return self.page < self.pages

    @property
    def next_num(self):
        """Number of the next page"""
        return self.page + 1

    def iter_pages(self, left_edge=2, left_current=2,
                   right_current=5, right_edge=2):
        u"""
        Iterates over the page numbers in the pagination.  The four
        parameters control the thresholds how many numbers should be produced
        from the sides.  Skipped page numbers are represented as `None`.
        This is how you could render such a pagination in the templates:

        .. sourcecode:: html+jinja

            {% macro render_pagination(pagination, endpoint) %}
              <div class=pagination>
              {%- for page in pagination.iter_pages() %}
                {% if page %}
                  {% if page != pagination.page %}
                    <a href="{{ url_for(endpoint, page=page) }}">{{ page }}</a>
                  {% else %}
                    <strong>{{ page }}</strong>
                  {% endif %}
                {% else %}
                  <span class=ellipsis>…</span>
                {% endif %}
              {%- endfor %}
              </div>
            {% endmacro %}
        """
        last = 0
        for num in xrange(1, self.pages + 1):
            if num <= left_edge or \
               (num > self.page - left_current - 1 and
                num < self.page + right_current) or \
               num > self.pages - right_edge:
                if last + 1 != num:
                    yield None
                yield num
                last = num